#include <windows.h>
#include <GL/glut.h>
#include <math.h>

void draw_triangle()
{
	glBegin(GL_LINE_LOOP);
	
	glVertex2d( 0.0,  1.0 / sqrt(3.0));
	glVertex2d( 0.5, -0.5 / sqrt(3.0));
	glVertex2d(-0.5, -0.5 / sqrt(3.0));

	glEnd();
}

void draw_sierpinski(int k)
{
	if (k == 0)
		draw_triangle();
	else
	{
		glPushMatrix();
		glTranslated(0.0, 0.5 / sqrt(3.0), 0.0);
		glScaled(0.5, 0.5, 0.5);
		draw_sierpinski(k-1);
		glPopMatrix();

		glPushMatrix();
		glTranslated(0.25, -0.25 / sqrt(3.0), 0.0);
		glScaled(0.5, 0.5, 0.5);
		draw_sierpinski(k-1);
		glPopMatrix();

		glPushMatrix();
		glTranslated(-0.25, -0.25 / sqrt(3.0), 0.0);
		glScaled(0.5, 0.5, 0.5);
		draw_sierpinski(k-1);
		glPopMatrix();
	}
}

void reshape(int w, int h)
{
	if (w <= 0) w = 1;
	if (h <= 0) h = 1;
	
	double aspect = w * 1.0 / h;

	glViewport(0, 0, w, h);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (w > h)
		gluOrtho2D(-aspect, aspect, -1.0, 1.0);
	else
		gluOrtho2D(-1.0, 1.0, -1.0/aspect, 1.0/aspect);
}

void display()
{
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	draw_sierpinski(8);

	glutSwapBuffers();
}

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
	glutInitWindowSize(512, 512);
	glutCreateWindow("Sierpinski Triangle");

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);

	glutMainLoop();

	return 0;
}