#include <windows.h>
#include <GL/glut.h>
#include <math.h>

#define M_PI 3.1415926535979

void draw_square(double r, double g, double b)
{
	glColor3d(r, g, b);
	glBegin(GL_QUADS);	
	glVertex2d(-1, -1);
	glVertex2d( 1, -1);
	glVertex2d( 1,  1);
	glVertex2d(-1,  1);
	glEnd();

	glColor3d(0,0,0);
	glLineWidth(2.0);
	glBegin(GL_LINE_LOOP);
	glVertex2d(-1, -1);
	glVertex2d( 1, -1);
	glVertex2d( 1,  1);
	glVertex2d(-1,  1);
	glEnd();
}

void draw_circle(double r, double g, double b)
{
	glColor3d(r, g, b);
	glBegin(GL_TRIANGLE_FAN);
	glVertex2d(0.0, 0.0);
	for(int i=0;i<256;i++)
	{
		double theta = 2 * M_PI * i / 256;
		double y = sin(theta);
		double x = cos(theta);
		glVertex2d(x,y);
	}
	glVertex2d(1.0, 0.0);
	glEnd();

	glColor3d(0.0, 0.0, 0.0);
	glLineWidth(2.0);
	glBegin(GL_LINE_LOOP);	
	for(int i=0;i<256;i++)
	{
		double theta = 2 * M_PI * i / 256;
		double y = sin(theta);
		double x = cos(theta);
		glVertex2d(x,y);
	}
	glEnd();
}

void draw_chair()
{
	glPushMatrix();
	glTranslated(0.1, 0.35, 0.0);
	glScaled(0.1, 0.35, 1.0);
	draw_square(0.0, 1.0, 1.0);
	glPopMatrix();

	glPushMatrix();
	glTranslated(0.3, 0.8, 0.0);
	glScaled(0.3, 0.1, 1.0);
	draw_square(0.0, 1.0, 1.0);
	glPopMatrix();

	glPushMatrix();
	glTranslated(0.7, 1.1, 0.0);
	glScaled(0.1, 1.1, 1.0);
	draw_square(0.0, 1.0, 1.0);
	glPopMatrix();
}

void draw_table()
{
	glPushMatrix();
	glTranslated(0.1, 0.55, 0.0);
	glScaled(0.1, 0.55, 1.0);
	draw_square(0.5, 0.5, 0.0);
	glPopMatrix();

	glPushMatrix();
	glTranslated(1.2, 1.2, 0.0);
	glScaled(1.2, 0.1, 1.0);
	draw_square(0.5, 0.5, 0.0);
	glPopMatrix();

	glPushMatrix();
	glTranslated(2.3, 0.55, 0.0);
	glScaled(0.1, 0.55, 1.0);
	draw_square(0.5, 0.5, 0.0);
	glPopMatrix();
}

void draw_tray()
{
	glPushMatrix();
	glTranslated(0.3, 0.3, 0.0);
	glScaled(0.2, 0.2, 0.2);
	draw_circle(1.0, 1.0, 0.0);
	glPopMatrix();

	glPushMatrix();
	glTranslated(0.6, 0.3, 0.0);
	glScaled(0.2, 0.2, 0.2);
	draw_circle(1.0, 0.0, 0.0);
	glPopMatrix();

	glPushMatrix();
	glTranslated(0.45, 0.15, 0.0);
	glScaled(0.45, 0.15, 1.0);
	draw_square(0.0, 1.0, 0.0);
	glPopMatrix();
}

void draw_table_and_tray()
{
	draw_table();

	glPushMatrix();
	glTranslated(0.8, 1.3, 0.0);
	draw_tray();
	glPopMatrix();
}

void draw_floor()
{
	glPushMatrix();
	glTranslated(0.0, -1.05, 0.0);
	glScaled(2.1, 0.05, 1.0);
	draw_square(0.0, 0.0, 0.0);
	glPopMatrix();
}

void draw_scene()
{
	glPushMatrix();
	glTranslated(-1.7, -1.0, 0.0);
	draw_table_and_tray();
	glPopMatrix();

	draw_floor();

	glPushMatrix();
	glTranslated(0.9, -1.0, 0.0);
	draw_chair();
	glPopMatrix();
}

void reshape(int w, int h)
{
	if (w <= 0) w = 1;
	if (h <= 0) h = 1;
	
	double aspect = w * 1.0 / h;

	glViewport(0, 0, w, h);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (w > h)
		gluOrtho2D(-aspect, aspect, -3.0, 3.0);
	else
		gluOrtho2D(-3.0, 3.0, -3.0/aspect, 3.0/aspect);
}

void display()
{
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	draw_scene();

	glutSwapBuffers();
}

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE);
	glutInitWindowSize(512, 512);
	glutCreateWindow("Table");

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);

	glutMainLoop();

	return 0;
}